/// @file 		enemy.h
/// @author		CloseToMetal
/// @version	1.0
/// @date		14/10/2014
///
///	@section	License
///
/// The MIT License (MIT)
/// 
/// Copyright (c) 2014 CloseToMetal
/// 
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
/// 
/// The above copyright notice and this permission notice shall be included in all
/// copies or substantial portions of the Software.
/// 
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.
#ifndef ENEMY_H_
#define ENEMY_H_

#include "player.h"
#include "tiles.h"
#include "utils.h"
#include "graphics.h"

#define MAX_PATROL_PATH_LENGTH 60
#define CHASE_STOP_PROBABILITY 12
/// Enemy actions
#define AI_MOVE_UP 0
#define AI_MOVE_RIGHT 1
#define AI_MOVE_DOWN 2
#define AI_MOVE_LEFT 3
#define AI_ATTACK_UP 4
#define AI_ATTACK_RIGHT 5
#define AI_ATTACK_DOWN 6
#define AI_ATTACK_LEFT 7
#define AI_STAND_GROUND 8
/// Enemy strategies
#define ST_FLEE 0
#define ST_ENGAGE 1
//#define ST_FOLLOW_SMELL 3
#define ST_PATROL 2
// Enemy Types
#define ESQUELETO 0
#define MURCIELAGO 1
#define TIRAMOCO 2

typedef struct moco
{
	unsigned char x;
	unsigned char y;
	unsigned char range;

	char speedX;
	char speedY;

	char *sprite;
	unsigned char damage;

} TMoco;

/// @brief 		Basic enemy structure.
typedef struct enemy 
{
	/// Coordinates of the position.
	unsigned char x;
	unsigned char y;
	//Enemy type
	unsigned char type;
	/// Pointer to the sprite address.
	char *sprite;
	char *spriteDeath;
	/// Look direction of the enemy (0-right, 64-bottom, 128-left, 192-top) to offset the sprite.
	unsigned char direction;
	//Turn to do actions
	unsigned char turn;
	/// Action and strategy selectors.
	unsigned char currentAction;
	unsigned char currentStrategy;
	/// Enemy game attributes.
	unsigned char hearThreshold;
	unsigned char hearRadius;
	unsigned char sightRadius;
	unsigned char path[MAX_PATROL_PATH_LENGTH];
	unsigned char pathNodes;
	unsigned char currentPathNode;
	char pathDir;
	unsigned char armor;
	unsigned char health;
	unsigned char strengthMax;
	unsigned char strengthMin;
	unsigned char critProb;
	// ¿Is the enemy alive?
	unsigned char alive;
	//Special hability
	TMoco moco;
	unsigned char redraw;
	unsigned char blocked;
	
} TEnemy;

/// @brief		Short-term Action processor.
/// @details	Carries out the current enemy's short term action by modifying
/// 			the status of the given enemy taking into account the currently
///				selected action.
/// @param		pEnemy A pointer to the enemy's structure.
/// @param		pPlayer A pointer to the player's structure.
void ExecuteAction(TEnemy *pEnemy, TPlayer *pPlayer, char *pPaletteHit, char *pPaletteHW);
/// @brief		Long-term Strategy processor.
/// @details	Carries out the current long term strategy of the enemy,
///				transitioning to the proper strategy if the conditions
///				are fulfilled or chooses the right short term action
///				for the current strategy and situation based on the
///				map and the player/enemy situation.
/// @param 		pEnemy Pointer to the enemy's structure to process the strategy.
/// @param		pPlayer Pointer to the player's structure.
/// @param		pMap Pointer to the map tile array.
/// @param		pMapWidth Width of the map in tiles.
/// @param		pTileSet Pointer to the tile set.
void ExecuteStrategy(TEnemy *pEnemy, TPlayer *pPlayer, char *pMap, char pMapWidth, TTile *pTileSet, TEnemy pEnemies[], char max_enemies);
/// @brief		Enemy drawing method
/// @details	Draws the specified enemy on screen.
/// @param		pEnemy A pointer to an enemy's structure to be drawn.
void DrawEnemy(TEnemy *pEnemy);

void DrawMoco(TEnemy *pEnemy);

#endif