/// @file       graphics.c
/// @author     CloseToMetal
/// @version    2.0
/// @date       14/10/2014
///
/// @section    License
///
/// The MIT License (MIT)
/// 
/// Copyright (c) 2014 CloseToMetal
/// 
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
/// 
/// The above copyright notice and this permission notice shall be included in all
/// copies or substantial portions of the Software.
/// 
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.

#include "graphics.h"
#include "cpcrslib.h"
#include "utils.h"

#define TILE_WIDTH 8
#define TILE_HEIGHT 8
#define TILE_WIDTH_BYTES TILE_WIDTH/2
#define TILE_HEIGHT_BYTES TILE_HEIGHT
#define TILE_SIZE_BYTES TILE_WIDTH_BYTES * TILE_HEIGHT_BYTES

#define SCREEN_WIDTH 20
#define SCREEN_HEIGHT 20

#define MAX_MARKABLE_TILES 20

unsigned char *mTileMap;
unsigned char mMapWidth;
unsigned char mMapHeight;
TTile *mTileSet;
unsigned char mMarkedTiles[MAX_MARKABLE_TILES*2];
unsigned char mNumMarkedTiles = 0;
unsigned char mMapMarked = 0;

void WaitVSYNC() 
{
    __asm 
        wait_vs:
            ld b, #0xF5
            in a,(c)
            rra
            jr nc, wait_vs
    __endasm;
}

void SetInks(const char *pPalette)
{
    unsigned char i;
    for (i = 0; i < 16; ++i)
        cpc_SetInk(i, pPalette[i]);
}
void SetColors (const char *pPalette)
{
    unsigned char i;
    for (i = 0; i < 16; ++i)
        cpc_SetColour(i, pPalette[i]);
}

void SetTileMap(unsigned char *pTileMap, unsigned char pWidth, unsigned char pHeight)
{
    mTileMap = pTileMap;
    mMapWidth = pWidth;
    mMapHeight = pHeight;
}
void SetTileSet(TTile *pTileSet)
{
    mTileSet = pTileSet;
}

void MarkTile(unsigned char pX, unsigned char pY)
{
    if ((mNumMarkedTiles >> 2) < MAX_MARKABLE_TILES)
    {
        mMarkedTiles[mNumMarkedTiles++] = pX;
        mMarkedTiles[mNumMarkedTiles++] = pY;
    }
}

void MarkAllTiles()
{
    mMapMarked = 1;
}

void DrawMarkedTiles(unsigned char pX, unsigned char pY, int pDir)
{
    unsigned char i;
    unsigned char j;
    unsigned char tile;
    unsigned int dir;

    if (mMapMarked)
    {
        DrawMap(pX, pY, pDir);
        mMapMarked = 0;
        mNumMarkedTiles = 0;
    }
    else
    {
        while (mNumMarkedTiles)
        {
            j = mMarkedTiles[--mNumMarkedTiles];
            i = mMarkedTiles[--mNumMarkedTiles];
            tile = getBit(mTileMap, (j * mMapWidth + i));
            dir = (int)(0xC000 + 0x50*(j % SCREEN_HEIGHT) + ((i % SCREEN_WIDTH) << 2));

            //if (tile == 1 && j < mMapHeight-1 && getBit(mTileMap, ((j+1) * mMapWidth + i)) == 0)
              //PutSprite4x8(mTileSet[3].sprite, pDir);
            //else
              PutSprite4x8(mTileSet[tile].sprite, dir);
        }
    }
}

void DrawMap(unsigned char pX, unsigned char pY, int pDir)
{
    unsigned char i;
    unsigned char j;
    unsigned char tile;

    unsigned char yLimit = (pY/SCREEN_HEIGHT * SCREEN_HEIGHT) + 20;
    unsigned char xLimit = (pX/SCREEN_WIDTH * SCREEN_WIDTH) + 20;
    
    for (i = pY/SCREEN_HEIGHT * SCREEN_HEIGHT; i < yLimit; ++i)
    {
        for (j = pX/SCREEN_WIDTH * SCREEN_WIDTH; j < xLimit; ++j)
        {
            tile = getBit(mTileMap, (i * mMapWidth + j));

            if (tile == 1 && i < mMapHeight-1 && getBit(mTileMap, ((i+1) * mMapWidth + j)) == 0)
              PutSprite4x8(mTileSet[3].sprite, pDir);
            else
              PutSprite4x8(mTileSet[tile].sprite, pDir);

            pDir += 4;
        }
    }    
}

char CheckSameRoom(unsigned char pX1, unsigned char pY1, unsigned char pX2, unsigned char pY2)
{
    unsigned char roomX = pX1 / SCREEN_WIDTH * SCREEN_WIDTH;
    unsigned char roomY = pY1 / SCREEN_HEIGHT * SCREEN_HEIGHT;

    return (pX2 >= roomX && pX2 < roomX + SCREEN_WIDTH && pY2 >= roomY && pY2 < roomY + SCREEN_HEIGHT);
}

void PutSprite4x8(char *pSprite, int pAddress)
{
    __asm
       POP  AF                 
       POP  HL                 
       POP  DE                 
       PUSH DE                 
       PUSH HL                 
       PUSH AF                 

       LD   A, #8              
       JP   dsa48_first_line 

    dsa48_next_line:
       EX   DE, HL             
       ADD  HL, BC             
       EX   DE, HL             

    dsa48_first_line:
       LD   BC, #0x800         
       LDI                     
       LDI                     
       LDI                     
       LDI                     

       DEC  A                  
       JP   NZ,dsa48_next_line

       RET                   
   __endasm;
}