/// @file 		graphics.h
/// @author		CloseToMetal
/// @version	2.0
/// @date		14/10/2014
///
///	@section	License
///
/// The MIT License (MIT)
/// 
/// Copyright (c) 2014 CloseToMetal
/// 
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
/// 
/// The above copyright notice and this permission notice shall be included in all
/// copies or substantial portions of the Software.
/// 
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.

#ifndef GRAPHICS_H_
#define GRAPHICS_H_

#include "tiles.h"

/// @brief		VSYNC wait.
/// @details	Keeps the program waiting to synchronize with the VSYNC signal.
///				Function extracted from CPCRSLIB.
void WaitVSYNC();

/// @brief		Inks setter.
/// @details	Sets the inks of the CPC to match the given palette.
/// @param		pPaleta The new palette of inks.
void SetInks(const char *pPaleta);
/// @brief		Colors setter.
/// @details	Sets the colors of the CPC to match the given palette.
/// @param 		pPaleta The new paletet of colors.
void SetColors (const char *pPaleta);

/// @brief		Tile Map setter.
/// @details 	Selects the tile map to be rendered on screen and its dimensions.
/// @param		pTileMap Pointer to the tile map.
/// @param		pMapWidth Width of the tile map.
/// @param		pMapHeight Height of the tile map.
void SetTileMap(unsigned char *pTileMap, unsigned char pMapWidth, unsigned char pMapHeight);
/// @brief		Tile Set setter.
/// @details  	Selects the tile set of the map.
/// @param		pTileSet Pointer to the tile set.
void SetTileSet(TTile *pTileSet);

/// @brief		Tile marker.
/// @details 	Marks the specified tile to be redrawn.
/// @param		pX The position on the x-axis of the tile.
/// @param		pY The position on the y-axis of the tile.
void MarkTile(unsigned char pX, unsigned char pY);
/// @brief		Full Map marker.
/// @details	Marks the full map to be redrawn.
void MarkAllTiles();
/// @brief		Marked tiles drawer.
/// @details	Redraws the marked tiles on screen.
/// @param		pX Current position of the player on the x-axis.
/// @param		pY Current position of the player on the y-axis.
/// @param		pDir Starting memory address for map drawing.
void DrawMarkedTiles(unsigned char pX, unsigned char pY, int pDir);
/// @brief		Map drawer.
/// @details	Draws a portion of the map on the screen based on the player
///				position. A portion of the map with a predetermined size is
///				selected to be drawn depending on the player position following
///				a room based approach in a Zelda-like fashion.
/// @param		pX Current position of the player on the x-axis.
/// @param		pY Current position of the player on the y-axis.
/// @param		pDir Starting memory address for map drawing.
void DrawMap(unsigned char pX, unsigned char pY, int pDir);

/// @brief		Same room checker.
/// @details	Checks if a certain position is in the same room as another one.
///				Two points are considered to be in the same position if they are
///				in the same 20x20 section of the map.
/// @param		pX1 The x coordinate of the first point.
/// @param		pY1 The y coordinate of the first point.
/// @param		pX2 The x coordinate of the second point.
/// @param		pY2 The y coordinate of the second point.
/// @return		False (0) if they are not in the same room, true (1) otherwise.
char CheckSameRoom(unsigned char pX1, unsigned char pY1, unsigned char pX2, unsigned char pY2);
/// @brief		8x8 pixels sprite drawing routine.
/// @details	Function for fast drawing of a 4x8 bytes sprite extracted from
///	 			the CPCTelera.
/// @param		pSprite The address of the sprite.
/// @param		pAddress The video RAM address to start drawing.
void PutSprite4x8(char *pSprite, int pAddress);

#endif