/// @file       randomfloor.h
/// @author     CloseToMetal
/// @version    1.0
/// @date       06/10/2014
///
/// @section    License
///
/// The MIT License (MIT)
/// 
/// Copyright (c) 2014 CloseToMetal
/// 
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
/// 
/// The above copyright notice and this permission notice shall be included in all
/// copies or substantial portions of the Software.
/// 
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.

#ifndef RANDOMFLOOR_H_
#define RANDOMFLOOR_H_

#include "utils.h"

/// @brief		Floor Initializer.
/// @details  	Initializes the new map.
/// @param		rows The height of the map.
/// @param		cols The width of the map.
/// @param		type Defines the way to initialize the map (to walls or grounds).
/// @param		x Coordinate x from which the method has to start the initialization. 
/// @param		y Coordinate y from which the method has to start the initialization.
/// @param		nTiles Number of tiles to compute. If 0, it completes the computation for the whole map.
void InitFloor(char *floor, unsigned int rows, unsigned int cols, unsigned int type, unsigned int x, unsigned int y, unsigned int nTiles);

/// @brief		Floor Generator.
/// @details  	Generates the composition of the map randomply.
/// @param		rows The height of the map.
/// @param		cols The width of the map.
/// @param		estado Indicates the part of the generation that has to be executed.
/// @param		x Coordinate x from which the method has to start the initialization. 
/// @param		y Coordinate y from which the method has to start the initialization.
/// @param		nTiles Number of tiles to compute. If 0, it completes the computation for the whole map.
char GenerateCavePart(char *floor, unsigned int rows, unsigned int cols, char estado, unsigned int j, unsigned int i, unsigned int nTiles);

/// @brief		Process the map to ensure connectivity.
/// @details  	Turns Unconnected ground tiles into walls and returns the number of walkable tiles.
/// @param		floor The whole matrix that defines the map.
/// @param 		auxFloor Auxiliar matrix needed to have the connected tiles.
/// @param		rows The height of the map.
/// @param		cols The width of the map.
unsigned int EnsureConnectivity(char *floor, char *auxFloor, unsigned int rows, unsigned int cols);

/// @brief		Generator of random positions where items can be placed.
/// @details  	Choose randomly a tile where a item can be placed.
/// @param		floor The whole matrix that defines the map.
/// @param		rows The height of the map.
/// @param		cols The width of the map.
unsigned int RandomPosition(char *floor, unsigned int rows, unsigned int cols);

/// @brief		Generator of random positions bounded from another position.
/// @details  	Choose randomly a tile where a item can be placed.
/// @param		floor The whole matrix that defines the map.
/// @param		rows The height of the map.
/// @param		cols The width of the map.
/// @param		x The whole matrix that defines the map.
/// @param		y The height of the map.
/// @param		lowerBound The width of the map.
/// @param		upperBound The whole matrix that defines the map.
unsigned int RandomPositionBound(char *floor, unsigned int rows, unsigned int cols, unsigned int x, unsigned int y, unsigned int lowerBound, unsigned int upperBound);

#endif